@@ -239,6 +239,11 @@ gda_connection_dispose (GObject *object)
 		cnc->priv->trans_status = NULL;
 	}
 
+        if (cnc->priv->client) {
+		g_object_unref (cnc->priv->client);
+		cnc->priv->client = NULL;
+	}
+
 	/* chain to parent class */
 	parent_class->dispose (object);
 }
@@ -675,7 +680,6 @@ gda_connection_get_provider_obj (GdaConnection *cnc)
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 	g_return_val_if_fail (cnc->priv, NULL);
 
-	g_assert (cnc->priv->provider_obj);
 	return cnc->priv->provider_obj;
 }
 
@@ -691,6 +695,8 @@ gda_connection_get_infos (GdaConnection *cnc)
 {
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 	g_return_val_if_fail (cnc->priv, NULL);
+	if (!cnc->priv->provider_obj)
+		return NULL;
 
 	return gda_server_provider_get_info (cnc->priv->provider_obj, cnc);
 }
@@ -708,6 +714,8 @@ gda_connection_get_server_version (GdaConnection *cnc)
 {
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 	g_return_val_if_fail (cnc->priv, NULL);
+	if (!cnc->priv->provider_obj)
+		return NULL;
 
 	return gda_server_provider_get_server_version (cnc->priv->provider_obj, cnc);
 }
@@ -726,6 +734,8 @@ gda_connection_get_database (GdaConnection *cnc)
 {
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 	g_return_val_if_fail (cnc->priv, NULL);
+	if (!cnc->priv->provider_obj)
+		return NULL;
 
 	return gda_server_provider_get_database (cnc->priv->provider_obj, cnc);
 }
@@ -1010,7 +1020,7 @@ gda_connection_add_event_string (GdaConnection *cnc, const gchar *str, ...)
 	return error;
 }
 
-/** TODO: This actually frees the input GList. That's very very unusual. murrayc.
+/** TODO: This actually frees the input GList. That's very very unusual. murrayc. */
 
 /**
  * gda_connection_add_events_list
@@ -1081,6 +1091,8 @@ gda_connection_change_database (GdaConnection *cnc, const gchar *name)
 {
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 	g_return_val_if_fail (name != NULL, FALSE);
+	if (!cnc->priv->provider_obj)
+		return FALSE;
 
 	return gda_server_provider_change_database (cnc->priv->provider_obj, cnc, name);
 }
@@ -1128,6 +1140,7 @@ gda_connection_execute_command (GdaConnection *cnc, GdaCommand *cmd,
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 	g_return_val_if_fail (cnc->priv, NULL);
 	g_return_val_if_fail (cmd != NULL, NULL);
+	g_return_val_if_fail (cnc->priv->provider_obj, NULL);
 
 	/* clean any previous connection events */
 	gda_connection_clear_events_list (cnc);
@@ -1169,6 +1182,10 @@ gda_connection_execute_command (GdaConnection *cnc, GdaCommand *cmd,
  * A connection must be specified, and, optionally, a result set. If not NULL,
  * the underlying provider should try to get the last insert ID for the given result set.
  *
+ * Beware however that the interpretation and usage of this value depends on the
+ * DBMS type being used, , see the <link linkend="limitations">limitations</link> 
+ * of each DBMS for more information.
+ *
  * Returns: a string representing the ID of the last inserted row, or NULL
  * if an error occurred or no row has been inserted. It is the caller's
  * reponsibility to free the returned string.
@@ -1178,6 +1195,7 @@ gda_connection_get_last_insert_id (GdaConnection *cnc, GdaDataModel *recset)
 {
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 	g_return_val_if_fail (cnc->priv, NULL);
+	g_return_val_if_fail (cnc->priv->provider_obj, NULL);
 
 	return gda_server_provider_get_last_insert_id (cnc->priv->provider_obj, cnc, recset);
 }
@@ -1231,9 +1249,12 @@ gda_connection_execute_select_command (GdaConnection *cnc, GdaCommand *cmd,
 
 		event = gda_connection_event_new (GDA_CONNECTION_EVENT_NOTICE);
 		if (nb > 1)
-			str = g_strdup_printf ("(%d rows)", nb);
+			str = g_strdup_printf (_("(%d rows)"), nb);
+		else if (nb >= 0)
+			str = g_strdup_printf (_("(%d row)"), nb);
 		else
-			str = g_strdup_printf ("(%d row)", nb);
+			str = g_strdup_printf (_("(unknown number of rows)"), nb);
+			
 		gda_connection_event_set_description (event, str);
 		g_free (str);
 		gda_connection_add_event (cnc, event);
@@ -1343,6 +1364,7 @@ gda_connection_begin_transaction (GdaConnection *cnc, const gchar *name, GdaTran
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 	g_return_val_if_fail (cnc->priv, FALSE);
+	g_return_val_if_fail (cnc->priv->provider_obj, FALSE);
 
 	retval = gda_server_provider_begin_transaction (cnc->priv->provider_obj, cnc, name, level, error);
 	if (retval)
@@ -1370,6 +1392,7 @@ gda_connection_commit_transaction (GdaConnection *cnc, const gchar *name, GError
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 	g_return_val_if_fail (cnc->priv, FALSE);
+	g_return_val_if_fail (cnc->priv->provider_obj, FALSE);
 
 	retval = gda_server_provider_commit_transaction (cnc->priv->provider_obj, cnc, name, error);
 	if (retval)
@@ -1398,6 +1421,7 @@ gda_connection_rollback_transaction (GdaConnection *cnc, const gchar *name, GErr
 
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 	g_return_val_if_fail (cnc->priv, FALSE);
+	g_return_val_if_fail (cnc->priv->provider_obj, FALSE);
 
 	retval = gda_server_provider_rollback_transaction (cnc->priv->provider_obj, cnc, name, error);
 	if (retval)
@@ -1421,6 +1445,7 @@ gda_connection_add_savepoint (GdaConnection *cnc, const gchar *name, GError **er
 {
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 	g_return_val_if_fail (cnc->priv, FALSE);
+	g_return_val_if_fail (cnc->priv->provider_obj, FALSE);
 	
 	return gda_server_provider_add_savepoint (cnc->priv->provider_obj, cnc, name, error);
 }
@@ -1440,7 +1465,8 @@ gda_connection_rollback_savepoint (GdaConnection *cnc, const gchar *name, GError
 {
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 	g_return_val_if_fail (cnc->priv, FALSE);
-	
+	g_return_val_if_fail (cnc->priv->provider_obj, FALSE);	
+
 	return gda_server_provider_rollback_savepoint (cnc->priv->provider_obj, cnc, name, error);
 }
 
@@ -1459,7 +1485,8 @@ gda_connection_delete_savepoint (GdaConnection *cnc, const gchar *name, GError *
 {
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 	g_return_val_if_fail (cnc->priv, FALSE);
-	
+	g_return_val_if_fail (cnc->priv->provider_obj, FALSE);
+
 	return gda_server_provider_delete_savepoint (cnc->priv->provider_obj, cnc, name, error);
 }
 
@@ -1497,6 +1524,7 @@ gda_connection_supports_feature (GdaConnection *cnc, GdaConnectionFeature featur
 {
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 	g_return_val_if_fail (cnc->priv, FALSE);
+	g_return_val_if_fail (cnc->priv->provider_obj, FALSE);
 
 	return gda_server_provider_supports_feature (cnc->priv->provider_obj, cnc, feature);
 }
@@ -1521,7 +1549,7 @@ gda_connection_supports_feature (GdaConnection *cnc, GdaConnectionFeature featur
  * <link linkend="libgda-provider-get-schema">get_schema() virtual method for providers</link> for more details.
  *
  * Returns: a #GdaDataModel containing the data required. The caller is responsible
- * of freeing the returned model using g_object_unref().
+ * for freeing the returned model using g_object_unref().
  */
 GdaDataModel *
 gda_connection_get_schema (GdaConnection *cnc,
@@ -1530,6 +1558,8 @@ gda_connection_get_schema (GdaConnection *cnc,
 			   GError **error)
 {
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+	g_return_val_if_fail (cnc->priv->provider_obj, NULL);
+
 	return gda_server_provider_get_schema (cnc->priv->provider_obj, cnc, schema, params, error);
 }
 
@@ -1567,6 +1597,7 @@ gda_connection_value_to_sql_string (GdaConnection *cnc, GValue *from)
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 	g_return_val_if_fail (cnc->priv, FALSE);
 	g_return_val_if_fail (from != NULL, FALSE);
+	g_return_val_if_fail (cnc->priv->provider_obj, FALSE);
 
 	/* execute the command on the provider */
 	return gda_server_provider_value_to_sql_string (cnc->priv->provider_obj, cnc, from);
