@@ -1,28 +1,32 @@
 /* gda-data-model-iter.c
  *
- * Copyright (C) 2005 - 2006 Vivien Malerba
+ * Copyright (C) 2005 - 2007 Vivien Malerba
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * This Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This Library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Library General Public
+ * License along with this Library; see the file COPYING.LIB.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include <glib/gi18n-lib.h>
 #include <string.h>
 #include "gda-data-model-iter.h"
 #include "gda-data-model.h"
+#include "gda-data-model-private.h"
 #include "gda-parameter.h"
+#include "gda-marshal.h"
+#include "gda-data-proxy.h"
+#include "gda-enums.h"
 
 /* 
  * Main static functions 
@@ -50,6 +54,7 @@ static void model_row_updated_cb (GdaDataModel *model, gint row, GdaDataModelIte
 static void model_row_removed_cb (GdaDataModel *model, gint row, GdaDataModelIter *iter);
 
 static void param_changed_cb (GdaParameterList *paramlist, GdaParameter *param);
+static void param_attr_changed_cb (GdaParameterList *paramlist, GdaParameter *param);
 
 #ifdef GDA_DEBUG
 static void gda_data_model_iter_dump (GdaDataModelIter *iter, guint offset);
@@ -61,12 +66,13 @@ static GObjectClass  *parent_class = NULL;
 /* signals */
 enum
 {
+	ROW_TO_CHANGE,
         ROW_CHANGED,
 	END_OF_DATA,
         LAST_SIGNAL
 };
 
-static gint gda_data_model_iter_signals[LAST_SIGNAL] = { 0, 0 };
+static gint gda_data_model_iter_signals[LAST_SIGNAL] = { 0, 0, 0 };
 
 /* properties */
 enum
@@ -97,7 +103,6 @@ GQuark gda_data_model_iter_error_quark (void)
 	return quark;
 }
 
-
 GType
 gda_data_model_iter_get_type (void)
 {
@@ -122,6 +127,26 @@ gda_data_model_iter_get_type (void)
 	return type;
 }
 
+static gboolean
+row_to_change_accumulator (GSignalInvocationHint *ihint,
+			   GValue *return_accu,
+			   const GValue *handler_return,
+			   gpointer data)
+{
+	gboolean thisvalue;
+
+	thisvalue = g_value_get_boolean (handler_return);
+	g_value_set_boolean (return_accu, thisvalue);
+
+	return thisvalue; /* stop signal if 'thisvalue' is FALSE */
+}
+
+static gboolean
+m_row_to_change (GdaDataModelIter *iter, gint row)
+{
+	return TRUE; /* defaults allows row to change */
+}
+
 static void
 gda_data_model_iter_class_init (GdaDataModelIterClass *class)
 {
@@ -130,6 +155,13 @@ gda_data_model_iter_class_init (GdaDataModelIterClass *class)
 
 	parent_class = g_type_class_peek_parent (class);
 
+	gda_data_model_iter_signals [ROW_TO_CHANGE] =
+                g_signal_new ("row_to_change",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (GdaDataModelIterClass, row_to_change),
+                              row_to_change_accumulator, NULL,
+                              gda_marshal_BOOLEAN__INT, G_TYPE_BOOLEAN, 1, G_TYPE_INT);
 	gda_data_model_iter_signals [ROW_CHANGED] =
                 g_signal_new ("row_changed",
                               G_TYPE_FROM_CLASS (object_class),
@@ -145,22 +177,26 @@ gda_data_model_iter_class_init (GdaDataModelIterClass *class)
                               NULL, NULL,
                               g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
 
+	class->row_to_change = m_row_to_change;
 	class->row_changed = NULL;
 	class->end_of_data = NULL;
 
 	object_class->dispose = gda_data_model_iter_dispose;
 	object_class->finalize = gda_data_model_iter_finalize;
 	paramlist_class->param_changed = param_changed_cb;
+	paramlist_class->param_attr_changed = param_attr_changed_cb;
 
 	/* Properties */
 	object_class->set_property = gda_data_model_iter_set_property;
 	object_class->get_property = gda_data_model_iter_get_property;
 	g_object_class_install_property (object_class, PROP_DATA_MODEL,
-					 g_param_spec_pointer ("data_model", "Data model for which the iter is for", NULL, 
+					 g_param_spec_object ("data_model", "Data model for which the iter is for", NULL,
+                                                               GDA_TYPE_DATA_MODEL, 
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE |
 								G_PARAM_CONSTRUCT_ONLY)));
 	g_object_class_install_property (object_class, PROP_FORCED_MODEL,
-					 g_param_spec_pointer ("forced_model", NULL, NULL, 
+					 g_param_spec_object ("forced_model", NULL, NULL, 
+                                                               GDA_TYPE_DATA_MODEL,
 							       (G_PARAM_READABLE | G_PARAM_WRITABLE)));
 	g_object_class_install_property (object_class, PROP_CURRENT_ROW,
 					 g_param_spec_int ("current_row", "Current represented row in the data model", 
@@ -189,8 +225,7 @@ gda_data_model_iter_init (GdaDataModelIter *iter)
 
 /**
  * gda_data_model_iter_new
- * @query: a #GdaQuery in which the new object will be
- * @type: the GDA type for the value
+ * @model: Model used to create the #GdaDataModelIter
  *
  * Creates a new GdaDataModelIter object which represents a value or a parameter.
  *
@@ -278,6 +313,47 @@ param_changed_cb (GdaParameterList *paramlist, GdaParameter *param)
 		((GdaParameterListClass *) parent_class)->param_changed (paramlist, param);
 }
 
+/*
+ * This function is called when a parameter in @paramlist has its attributes changed
+ * to make sure the change is propagated to the GdaDataModel
+ * paramlist is an iter for
+ */
+static void
+param_attr_changed_cb (GdaParameterList *paramlist, GdaParameter *param)
+{
+	GdaDataModelIter *iter;
+	gint col;
+	gboolean toset;
+
+	iter = GDA_DATA_MODEL_ITER (paramlist);
+	if (iter->priv->keep_param_changes ||
+	    (iter->priv->row < 0))
+		return;
+
+	if (!GDA_IS_DATA_PROXY (iter->priv->data_model))
+		return;
+
+	g_signal_handler_block (iter->priv->data_model, iter->priv->model_changes_signals [0]);
+	g_signal_handler_block (iter->priv->data_model, iter->priv->model_changes_signals [1]);
+
+	/* propagate the value update to the data model */
+	col = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (param), "model_col")) - 1;
+	g_return_if_fail (col >= 0);
+
+	g_object_get (G_OBJECT (param), "use-default-value", &toset, NULL);
+	if (toset && gda_parameter_get_exists_default_value (param))
+		gda_data_proxy_alter_value_attributes (GDA_DATA_PROXY (iter->priv->data_model), 
+						       iter->priv->row, col, 
+						       GDA_VALUE_ATTR_CAN_BE_DEFAULT | GDA_VALUE_ATTR_IS_DEFAULT);
+	
+	g_signal_handler_unblock (iter->priv->data_model, iter->priv->model_changes_signals [0]);
+	g_signal_handler_unblock (iter->priv->data_model, iter->priv->model_changes_signals [1]);
+
+	/* for the parent class */
+	if (((GdaParameterListClass *) parent_class)->param_attr_changed)
+		((GdaParameterListClass *) parent_class)->param_attr_changed (paramlist, param);
+}
+
 static void 
 destroyed_object_cb (GdaObject *obj, GdaDataModelIter *iter)
 {
@@ -345,7 +421,6 @@ gda_data_model_iter_set_property (GObject *object,
 				  GParamSpec *pspec)
 {
 	GdaDataModelIter *iter;
-	gpointer ptr;
 
 	iter = GDA_DATA_MODEL_ITER (object);
 	if (iter->priv) {
@@ -357,7 +432,7 @@ gda_data_model_iter_set_property (GObject *object,
 			GdaDict *dict;
 			GdaColumn *column;
 
-			ptr = g_value_get_pointer (value);
+			GObject* ptr = g_value_get_object (value);
 			g_return_if_fail (ptr && GDA_IS_DATA_MODEL (ptr));
 			model = GDA_DATA_MODEL (ptr);
 			
@@ -372,11 +447,16 @@ gda_data_model_iter_set_property (GObject *object,
 				param = (GdaParameter *) g_object_new (GDA_TYPE_PARAMETER, "dict", dict,
 								       "g_type", 
 								       gda_column_get_g_type (column), NULL);
+
+				gda_parameter_set_not_null (param, !gda_column_get_allow_null (column));
 				str = gda_column_get_title (column);
 				if (!str)
 					str = gda_column_get_name (column);
 				if (str)
 					gda_object_set_name (GDA_OBJECT (param), str);
+				if (gda_column_get_default_value (column) || 
+				    gda_column_get_auto_increment (column))
+					gda_parameter_set_exists_default_value (param, TRUE);
 				gda_parameter_list_add_param ((GdaParameterList *) iter, param);
 				g_object_set_data (G_OBJECT (param), "model_col", GINT_TO_POINTER (col + 1));
 				g_object_unref (param);
@@ -384,8 +464,8 @@ gda_data_model_iter_set_property (GObject *object,
 							    G_CALLBACK (destroyed_param_cb), iter);
 			}
 		}
-		case PROP_FORCED_MODEL:
-			ptr = g_value_get_pointer (value);
+		case PROP_FORCED_MODEL: {
+			GdaDataModel* ptr = g_value_get_object (value);
 			g_return_if_fail (ptr && GDA_IS_DATA_MODEL (ptr));
 
 			if (iter->priv->data_model) {
@@ -403,13 +483,14 @@ gda_data_model_iter_set_property (GObject *object,
 			iter->priv->model_changes_signals [1] = g_signal_connect (G_OBJECT (ptr), "row_removed",
 										  G_CALLBACK (model_row_removed_cb), iter);
 			break;
+                }
 		case PROP_CURRENT_ROW:
 			if (iter->priv->row != g_value_get_int (value)) {
-				iter->priv->row = g_value_get_int (value);
-				g_signal_emit (G_OBJECT (iter),
-					       gda_data_model_iter_signals[ROW_CHANGED],
-					       0, iter->priv->row);
-			}
+                                iter->priv->row = g_value_get_int (value);
+                                g_signal_emit (G_OBJECT (iter),
+                                               gda_data_model_iter_signals[ROW_CHANGED],
+                                               0, iter->priv->row);
+                        }
 			break;
 		case PROP_UPDATE_MODEL:
 			iter->priv->keep_param_changes = ! g_value_get_boolean (value);
@@ -430,7 +511,7 @@ gda_data_model_iter_get_property (GObject *object,
 		switch (param_id) {
 		case PROP_DATA_MODEL:
 		case PROP_FORCED_MODEL:
-			g_value_set_pointer (value, iter->priv->data_model);
+			g_value_set_object (value, G_OBJECT (iter->priv->data_model));
 			break;
 		case PROP_CURRENT_ROW:
 			g_value_set_int (value, iter->priv->row);
@@ -442,6 +523,35 @@ gda_data_model_iter_get_property (GObject *object,
 	}
 }
 
+/**
+ * gda_data_model_iter_can_be_moved
+ * @iter: a #GdaDataModelIter object
+ *
+ * Tells if @iter can point to another row. Note the @iter by itself will not refuse
+ * a row change, but that the row change may be refused by another object using
+ * @iter.
+ *
+ * Returns: TRUE if the row represented by @iter can be changed
+ */
+gboolean
+gda_data_model_iter_can_be_moved (GdaDataModelIter *iter)
+{
+	gboolean move_ok = TRUE;
+
+	g_return_val_if_fail (GDA_IS_DATA_MODEL_ITER (iter), FALSE);
+	g_return_val_if_fail (iter->priv, FALSE);
+
+	if (!gda_data_model_iter_is_valid (iter))
+		return TRUE;
+
+	/* optionally validate the row change */
+	g_signal_emit (G_OBJECT (iter),
+		       gda_data_model_iter_signals[ROW_TO_CHANGE],
+		       0, iter->priv->row, &move_ok);
+
+	return move_ok;
+}
+
 /**
  * gda_data_model_iter_set_at_row
  * @iter: a #GdaDataModelIter object
@@ -451,7 +561,7 @@ gda_data_model_iter_get_property (GObject *object,
  *
  * If @row < 0 then @iter is not bound to any row of the data model it iters through.
  *
- * Returns: TRUE if no error occures
+ * Returns: TRUE if no error occurred
  */
 gboolean
 gda_data_model_iter_set_at_row (GdaDataModelIter *iter, gint row)
@@ -479,7 +589,7 @@ gda_data_model_iter_set_at_row (GdaDataModelIter *iter, gint row)
  * Moves @iter one row further than where it already is (synchronizes the values of the parameters in @iter 
  * with the values at the new row).
  *
- * Returns: TRUE if no error occures
+ * Returns: TRUE if no error occurred
  */
 gboolean
 gda_data_model_iter_move_next (GdaDataModelIter *iter)
@@ -497,7 +607,7 @@ gda_data_model_iter_move_next (GdaDataModelIter *iter)
  * Moves @iter one row before where it already is (synchronizes the values of the parameters in @iter 
  * with the values at the new row).
  *
- * Returns: TRUE if no error occures
+ * Returns: TRUE if no error occurred
  */
 gboolean
 gda_data_model_iter_move_prev (GdaDataModelIter *iter)
@@ -542,10 +652,6 @@ gda_data_model_iter_invalidate_contents (GdaDataModelIter *iter)
 	iter->priv->keep_param_changes = TRUE;
 	list = GDA_PARAMETER_LIST (iter)->parameters;
 	while (list) {
-		/* GdaParameter *tmp; */
-/* 		g_object_get (G_OBJECT (list->data), "simple_bind", &tmp, NULL); */
-/* 		if (! tmp) */
-/* 			gda_parameter_declare_invalid (GDA_PARAMETER (list->data)); */
 		gda_parameter_declare_invalid (GDA_PARAMETER (list->data));
 		list = g_slist_next (list);
 	}
@@ -556,7 +662,7 @@ gda_data_model_iter_invalidate_contents (GdaDataModelIter *iter)
  * gda_data_model_iter_is_valid
  * @iter: a #GdaDataModelIter object
  *
- * Tells if @iter is a valid iterator.
+ * Tells if @iter is a valid iterator (if it actually corresponds to a valid row in the model)
  *
  * Returns: TRUE if @iter is valid
  */
@@ -618,14 +724,16 @@ gda_data_model_iter_dump (GdaDataModelIter *iter, guint offset)
 
 	g_return_if_fail (GDA_IS_DATA_MODEL_ITER (iter));
 	dict = gda_object_get_dict (GDA_OBJECT (iter));
-	
+       
         /* string for the offset */
         str = g_new0 (gchar, offset+1);
 	memset (str, ' ', offset);
 
         /* dump */
         if (iter->priv) {
-		g_print ("Iter %p\n", iter);
+		GdaObjectClass *plist_class;
+		plist_class = g_type_class_peek (GDA_TYPE_PARAMETER_LIST);
+		(plist_class->dump) ((GdaObject *) iter, offset);
 	}
         else
                 g_print ("%s" D_COL_ERR "Using finalized object %p" D_COL_NOR, str, iter);
