@@ -2,20 +2,20 @@
  *
  * Copyright (C) 2003 - 2007 Vivien Malerba
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * This Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This Library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Library General Public
+ * License along with this Library; see the file COPYING.LIB.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include "gda-dict-field.h"
@@ -82,8 +82,7 @@ enum
 {
 	PROP_0,
 	PROP_DB_TABLE,
-	PROP_PLUGIN,
-	PROP_DICT_TYPE
+	PROP_PLUGIN
 };
 
 
@@ -456,7 +455,7 @@ gda_dict_field_get_scale (GdaDictField *field)
  * gda_dict_field_get_constraints
  * @field: a #GdaDictField  object
  *
- * Get all the constraints which impact the given field. Constraints are of several type:
+ * Get all the constraints which affect the given field. Constraints are of several type:
  * NOT NULL, primary key, foreign key, check constrains
  *
  * Returns: a new list of #GdaDictConstraint objects
