@@ -1,5 +1,5 @@
 /* GDA library
- * Copyright (C) 1998 - 2006 The GNOME Foundation.
+ * Copyright (C) 1998 - 2007 The GNOME Foundation.
  *
  * AUTHORS:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
@@ -31,7 +31,6 @@
 #include <libgda/gda-data-model.h>
 #include <libgda/gda-data-model-index.h>
 #include <libgda/gda-quark-list.h>
-#include <libgda/gda-transaction.h>
 #include <libgda/gda-client.h>
 
 G_BEGIN_DECLS
@@ -72,6 +71,15 @@ struct _GdaServerProviderInfo {
 	 * to write "SELECT field..."
 	 */
 	gboolean       supports_prefixed_fields;
+
+	/* TRUE if non lower case identifiers must be surrounded by double quotes to distinguish them
+	 * with their lower case equivalent, that is TRUE if by default the non lower case identifiers
+	 * are converted into lower case.
+	 */
+	gboolean       quote_non_lc_identifiers;
+
+	/* reserved for extensions */
+	gboolean       reserved[20];
 };
 
 struct _GdaServerProvider {
@@ -119,6 +127,10 @@ struct _GdaServerProviderClass {
 	const gchar           *(*get_def_dbms_type) (GdaServerProvider *provider,
 						     GdaConnection *cnc,
 						     GType g_type);
+	gchar                 *(*escape_string)     (GdaServerProvider *provider,
+						     GdaConnection *cnc, const gchar *str);
+	gchar                 *(*unescape_string)   (GdaServerProvider *provider,
+						     GdaConnection *cnc, const gchar *str);
 
 	/* connections management */
 	gboolean               (* open_connection) (GdaServerProvider *provider,
@@ -150,6 +162,10 @@ struct _GdaServerProviderClass {
 						     GdaConnection *cnc,
 						     GdaCommand *cmd,
 						     GdaParameterList *params);
+	GdaObject              *(* execute_query) (GdaServerProvider *provider,
+						   GdaConnection *cnc,
+						   GdaQuery *query,
+						   GdaParameterList *params);
 	char                   *(* get_last_insert_id) (GdaServerProvider *provider,
 							GdaConnection *cnc,
 							GdaDataModel *recset);
@@ -157,25 +173,30 @@ struct _GdaServerProviderClass {
 	/* transactions */
 	gboolean                (* begin_transaction) (GdaServerProvider *provider,
 						       GdaConnection *cnc,
-						       GdaTransaction *xaction);
+						       const gchar *name, GdaTransactionIsolation level,
+						       GError **error);
 	gboolean                (* commit_transaction) (GdaServerProvider *provider,
 							GdaConnection *cnc,
-							GdaTransaction *xaction);
+							const gchar *name, GError **error);
 	gboolean                (* rollback_transaction) (GdaServerProvider *provider,
 							  GdaConnection *cnc,
-							  GdaTransaction *xaction);
-
-	/* blobs */
-	GdaBlob                *(* create_blob) (GdaServerProvider *provider,
-						 GdaConnection *cnc);
-
-	GdaBlob                *(* fetch_blob) (GdaServerProvider *provider,
-						GdaConnection *cnc, const gchar *sql_id);
+							  const gchar *name, GError **error);
+	gboolean                (* add_savepoint)      (GdaServerProvider *provider,
+							GdaConnection *cnc,
+							const gchar *name, GError **error);
+	gboolean                (* rollback_savepoint) (GdaServerProvider *provider,
+							GdaConnection *cnc,
+							const gchar *name, GError **error);
+	gboolean                (* delete_savepoint)   (GdaServerProvider *provider,
+							GdaConnection *cnc,
+							const gchar *name, GError **error);
 
+	/* future extensions */
 	gpointer                   reserved1; /* is_reserved_word() ? */
 	gpointer                   reserved2; /* ping_or_reconnect() ? */
-	gpointer                   reserved3;
+	gpointer                   reserved3; /* execute () ? */
 	gpointer                   reserved4;
+	gpointer                   reserved5;
 };
 
 GType                  gda_server_provider_get_type (void);
@@ -212,7 +233,10 @@ gchar                 *gda_server_provider_value_to_sql_string   (GdaServerProvi
 const gchar           *gda_server_provider_get_default_dbms_type (GdaServerProvider *provider,
 								  GdaConnection *cnc,
 								  GType type);
-
+gchar                 *gda_server_provider_escape_string         (GdaServerProvider *provider,
+								  GdaConnection *cnc, const gchar *str);
+gchar                 *gda_server_provider_unescape_string       (GdaServerProvider *provider,
+								  GdaConnection *cnc, const gchar *str);
 
 /* connections management */
 gboolean               gda_server_provider_open_connection  (GdaServerProvider *provider,
@@ -220,8 +244,7 @@ gboolean               gda_server_provider_open_connection  (GdaServerProvider *
 							     GdaQuarkList *params,
 							     const gchar *username,
 							     const gchar *password);
-gboolean               gda_server_provider_reset_connection (GdaServerProvider *provider,
-							     GdaConnection *cnc);
+
 gboolean               gda_server_provider_close_connection (GdaServerProvider *provider,
 							     GdaConnection *cnc);
 
@@ -247,6 +270,10 @@ GList        *gda_server_provider_execute_command    (GdaServerProvider *provide
 						      GdaConnection *cnc,
 						      GdaCommand *cmd,
 						      GdaParameterList *params);
+GdaObject    *gda_server_provider_execute_query    (GdaServerProvider *provider,
+						    GdaConnection *cnc,
+						    GdaQuery *query,
+						    GdaParameterList *params);
 gchar        *gda_server_provider_get_last_insert_id (GdaServerProvider *provider,
 						      GdaConnection *cnc,
 						      GdaDataModel *recset);
@@ -254,19 +281,20 @@ gchar        *gda_server_provider_get_last_insert_id (GdaServerProvider *provide
 /* transactions */
 gboolean      gda_server_provider_begin_transaction    (GdaServerProvider *provider,
 							GdaConnection *cnc,
-							GdaTransaction *xaction);
+							const gchar *name, GdaTransactionIsolation level,
+							GError **error);
 gboolean      gda_server_provider_commit_transaction   (GdaServerProvider *provider,
 							GdaConnection *cnc,
-							GdaTransaction *xaction);
+							const gchar *name, GError **error);
 gboolean      gda_server_provider_rollback_transaction (GdaServerProvider *provider,
 							GdaConnection *cnc,
-							GdaTransaction *xaction);
-
-/* blobs */
-GdaBlob      *gda_server_provider_create_blob          (GdaServerProvider *provider,
-							GdaConnection *cnc);
-GdaBlob      *gda_server_provider_fetch_blob_by_id     (GdaServerProvider *provider,
-							GdaConnection *cnc, const gchar *sql_id);
+							const gchar *name, GError **error);
+gboolean      gda_server_provider_add_savepoint        (GdaServerProvider *provider, GdaConnection *cnc, 
+							const gchar *name, GError **error);
+gboolean      gda_server_provider_rollback_savepoint   (GdaServerProvider *provider, GdaConnection *cnc, 
+							const gchar *name, GError **error);
+gboolean      gda_server_provider_delete_savepoint     (GdaServerProvider *provider, GdaConnection *cnc, 
+							const gchar *name, GError **error);
 
 G_END_DECLS
 
