@@ -194,10 +194,23 @@ set_from_string (GValue *value, const gchar *as_string)
 		else
 			g_date_free (gdate);
 	}
-	else if (type == GDA_TYPE_NULL)
+	else if (type == GDA_TYPE_NULL) {
 		gda_value_set_null (value);
-	else if (type == G_TYPE_ULONG)
-		g_value_set_ulong (value, gda_g_type_from_string (as_string));
+		retval = TRUE;
+	}
+	else if (type == G_TYPE_ULONG) {
+		if (gda_g_type_from_string (as_string) != 0) 
+			g_value_set_ulong (value, gda_g_type_from_string (as_string));
+		else {
+			gulong ulvalue;
+
+			ulvalue = strtoul (as_string, endptr, 10);
+			if (*as_string!=0 && **endptr==0) {
+				g_value_set_ulong (value, ulvalue);
+				retval = TRUE;
+			}
+		}
+	}
 
 	return retval;
 }
@@ -1154,9 +1167,9 @@ gda_value_new_from_string (const gchar *as_string, GType type)
 
 /**
  * gda_value_new_from_xml
- * @node: a XML node representing the value.
+ * @node: an XML node representing the value.
  *
- * Creates a GValue from a XML representation of it. That XML
+ * Creates a GValue from an XML representation of it. That XML
  * node corresponds to the following string representation:
  *    &lt;value type="gdatype"&gt;value&lt;/value&gt;
  *
@@ -1723,10 +1736,18 @@ gda_value_stringify (const GValue *value)
 			if (type == G_TYPE_DATE) {
 				GDate *date;
 				date = (GDate *) g_value_get_boxed (value);
-				return g_strdup_printf ("%04u-%02u-%02u",
-							g_date_get_year (date),
-							g_date_get_month (date),
-							g_date_get_day (date));
+				if (date) {
+					if (g_date_valid (date))
+						return g_strdup_printf ("%04u-%02u-%02u",
+									g_date_get_year (date),
+									g_date_get_month (date),
+									g_date_get_day (date));
+					else
+						return g_strdup_printf ("%04u-%02u-%02u",
+									date->year, date->month, date->day);
+				}
+				else
+					return g_strdup ("0000-00-00");
 			}
 			else
 				return g_strdup ("");
@@ -1934,14 +1955,15 @@ gda_value_compare (const GValue *value1, const GValue *value2)
 		return (g_value_get_uint (value1) > g_value_get_uint (value2)) ? 1 : 
 			((g_value_get_uint (value1) == g_value_get_uint (value2)) ? 0 : -1);
 
-		return g_value_get_uint (value1) - g_value_get_uint(value2);
-
 
 	g_warning ("%s() cannot handle values of type %s", __FUNCTION__, g_type_name (G_VALUE_TYPE (value1)));
 
 	return 0;
 }
 
+#define _value_is_null(x) (!(x) || (G_VALUE_TYPE (x) == GDA_TYPE_NULL) || \
+			   ((G_VALUE_TYPE (x) == G_TYPE_STRING) && !g_value_get_string (x)))
+
 /**
  * gda_value_compare_ext
  * @value1: a #GValue to compare.
@@ -1959,16 +1981,16 @@ gda_value_compare_ext (const GValue *value1, const GValue *value2)
 	if (value1 == value2)
 		return 0;
 
-	if (!value1 || (G_VALUE_TYPE (value1) == GDA_TYPE_NULL)) {
+	if (_value_is_null (value1)) {
 		/* value1 represents a NULL value */
-		if (! value2 || (G_VALUE_TYPE (value2) == GDA_TYPE_NULL))
+		if (_value_is_null (value2))
 			return 0;
 		else
 			return 1;
 	}
 	else {
 		/* value1 does not represents a NULL value */
-		if (! value2 || (G_VALUE_TYPE (value2) == GDA_TYPE_NULL))
+		if (_value_is_null (value2))
 			return -1;
 		else
 			return gda_value_compare (value1, value2);
@@ -2005,7 +2027,7 @@ to_string (const GValue *value)
  * gda_value_to_xml
  * @value: a #GValue.
  *
- * Serializes the given #GValue to a XML node string.
+ * Serializes the given #GValue to an XML node string.
  *
  * Returns: the XML node. Once not needed anymore, you should free it.
  */
